@@ -1,23 +1,16 @@
 #!/usr/bin/env python
-# $Id: Filters.py,v 1.23 2005/01/03 18:08:15 tavis_rudd Exp $
+# $Id: Filters.py,v 1.32 2007/03/29 19:31:15 tavis_rudd Exp $
 """Filters for the #filter directive; output filters Cheetah's $placeholders .
 
-Filters may now be used standalone, for debugging or for use outside Cheetah.
-Class DummyTemplate, instance _dummyTemplateObj and class NoDefault exist only
-for this, to provide a default argument for the filter constructors (which
-would otherwise require a real template object).  
-
 Meta-Data
 ================================================================================
 Author: Tavis Rudd <tavis@damnsimple.com>
-Version: $Revision: 1.23 $
+Version: $Revision: 1.32 $
 Start Date: 2001/08/01
-Last Revision Date: $Date: 2005/01/03 18:08:15 $
+Last Revision Date: $Date: 2007/03/29 19:31:15 $
 """
 __author__ = "Tavis Rudd <tavis@damnsimple.com>"
-__revision__ = "$Revision: 1.23 $"[11:-2]
-
-from StringIO import StringIO # not cStringIO because of unicode support
+__revision__ = "$Revision: 1.32 $"[11:-2]
 
 # Additional entities WebSafe knows how to transform.  No need to include
 # '<', '>' or '&' since those will have been done already.
@@ -26,74 +19,48 @@ webSafeEntities = {' ': '&nbsp;', '"': '&quot;'}
 class Error(Exception):
     pass
 
-class NoDefault:
-    pass
-
-
-class DummyTemplate:
-    """Fake template class to allow filters to be used standalone.
-
-    This is provides only the level of Template compatibility required by the
-    standard filters.  Namely, the get-settings interface works but there are
-    no settings.  Other aspects of Template are not implemented.
-    """
-    def setting(self, name, default=NoDefault):
-        if default is NoDefault:
-            raise KeyError(name)
-        else:
-            return default
-
-    def settings(self):
-        return {}
-
-_dummyTemplateObj = DummyTemplate()
-
-
 ##################################################
 ## BASE CLASS
 
-class Filter:
+class Filter(object):
     """A baseclass for the Cheetah Filters."""
     
-    def __init__(self, templateObj=_dummyTemplateObj):
-        """Setup a ref to the templateObj.  Subclasses should call this method.
+    def __init__(self, template=None):
+        """Setup a reference to the template that is using the filter instance.
+        This reference isn't used by any of the standard filters, but is
+        available to Filter subclasses, should they need it.
+        
+        Subclasses should call this method.
         """
-        self.setting = templateObj.setting
-        self.settings = templateObj.settings
-
-    def generateAutoArgs(self):
+        self.template = template
         
-        """This hook allows the filters to generate an arg-list that will be
-        appended to the arg-list of a $placeholder tag when it is being
-        translated into Python code during the template compilation process. See
-        the 'Pager' filter class for an example."""
-        
-        return ''
-        
-    def filter(self, val, **kw):
-        
-        """Reimplement this method if you  want more advanced filterting."""
-        
-        return str(val)
+    def filter(self, val,
+               #encoding='utf8',
+               encoding=None,
+               str=str, 
+               **kw):
+        """Pass Unicode strings through unmolested, unless an encoding is specified.
+        """
+        if isinstance(val, unicode):
+            if encoding:
+                filtered = val.encode(encoding)
+            else:
+                filtered = val
+        elif val is None:
+            filtered = ''
+        else:
+            filtered = str(val)
+        return filtered
 
+RawOrEncodedUnicode = Filter
 
 ##################################################
 ## ENHANCED FILTERS
 
-#####
-class ReplaceNone(Filter):
-    def filter(self, val, **kw):
-        
-        """Replace None with an empty string.  Reimplement this method if you
-        want more advanced filterting."""
-        
-        if val is None:
-            return ''
-        return str(val)
-#####
 class EncodeUnicode(Filter):
-    def filter(self, val, encoding='utf8',
-               str=str, type=type, unicodeType=type(u''),
+    def filter(self, val,
+               encoding='utf8',
+               str=str,
                **kw):
         """Encode Unicode strings, by default in UTF-8.
 
@@ -105,77 +72,28 @@ class EncodeUnicode(Filter):
         ... filter='EncodeUnicode')
         >>> print t
         """
-        if type(val)==unicodeType:
+        if isinstance(val, unicode):
             filtered = val.encode(encoding)
         elif val is None:
             filtered = ''
         else:
             filtered = str(val)
         return filtered
-#####
+
 class MaxLen(Filter):
     def filter(self, val, **kw):
         """Replace None with '' and cut off at maxlen."""
-        output = Filter.filter(self, val, **kw)
+        
+    	output = super(MaxLen, self).filter(val, **kw)
         if kw.has_key('maxlen') and len(output) > kw['maxlen']:
             return output[:kw['maxlen']]
         return output
 
-
-#####
-class Pager(Filter):
-    def __init__(self, templateObj=_dummyTemplateObj):
-        Filter.__init__(self, templateObj)
-        self._IDcounter = 0
-        
-    def buildQString(self,varsDict, updateDict):
-        finalDict = varsDict.copy()
-        finalDict.update(updateDict)
-        qString = '?'
-        for key, val in finalDict.items():
-            qString += str(key) + '=' + str(val) + '&'
-        return qString
-
-    def generateAutoArgs(self):
-        ID = str(self._IDcounter)
-        self._IDcounter += 1
-        return ', trans=trans, ID=' + ID
-    
-    def filter(self, val, **kw):
-        """Replace None with '' and cut off at maxlen."""
-        output = Filter.filter(self, val, **kw)
-        if kw.has_key('trans') and kw['trans']:
-            ID = kw['ID']
-            marker = kw.get('marker', '<split>')
-            req = kw['trans'].request()
-            URI = req.environ()['SCRIPT_NAME'] + req.environ()['PATH_INFO']
-            queryVar = 'pager' + str(ID) + '_page'
-            fields = req.fields()
-            page = int(fields.get( queryVar, 1))
-            pages = output.split(marker)
-            output = pages[page-1]
-            output += '<BR>'
-            if page > 1:
-                output +='<A HREF="' + URI + self.buildQString(fields, {queryVar:max(page-1,1)}) + \
-                          '">Previous Page</A>&nbsp;&nbsp;&nbsp;'
-            if page < len(pages):
-                output += '<A HREF="' + URI + self.buildQString(
-                    fields,
-                    {queryVar:
-                     min(page+1,len(pages))}) + \
-                     '">Next Page</A>' 
-
-            return output
-        return output
-
-
-#####
 class WebSafe(Filter):
     """Escape HTML entities in $placeholders.
     """
     def filter(self, val, **kw):
-        # Do the default conversion.
-        s = Filter.filter(self, val, **kw)
+    	s = super(WebSafe, self).filter(val, **kw)
         # These substitutions are copied from cgi.escape().
         s = s.replace("&", "&amp;") # Must be done first!
         s = s.replace("<", "&lt;")
@@ -194,7 +112,6 @@ class WebSafe(Filter):
         return s
 
 
-#####
 class Strip(Filter):
     """Strip leading/trailing whitespace but preserve newlines.
 
@@ -211,23 +128,22 @@ class Strip(Filter):
     with the proposed #sed directive (which has not been ratified yet.)
     """
     def filter(self, val, **kw):
-        s = Filter.filter(self, val, **kw)
-        result = StringIO()
+    	s = super(Strip, self).filter(val, **kw)
+        result = []
         start = 0   # The current line will be s[start:end].
         while 1: # Loop through each line.
             end = s.find('\n', start)  # Find next newline.
             if end == -1:  # If no more newlines.
                 break
             chunk = s[start:end].strip()
-            result.write(chunk)
-            result.write('\n')
+            result.append(chunk)
+            result.append('\n')
             start = end + 1
         # Write the unfinished portion after the last newline, if any.
         chunk = s[start:].strip()
-        result.write(chunk)
-        return result.getvalue()
+        result.append(chunk)
+        return "".join(result)
 
-#####
 class StripSqueeze(Filter):
     """Canonicalizes every chunk of whitespace to a single space.
 
@@ -235,8 +151,8 @@ class StripSqueeze(Filter):
     input is joined into one ling line with NO trailing newline.
     """
     def filter(self, val, **kw):
-        s = Filter.filter(self, val, **kw)
-        s = val.split()
+    	s = super(StripSqueeze, self).filter(val, **kw)
+        s = s.split()
         return " ".join(s)
     
 ##################################################
